@@ -813,7 +813,11 @@ parse_json(VALUE clas, char *json, int given, int allocated) {
     }
 #endif
     // last arg is free func void* func(void*)
+#if HAS_DATA_OBJECT_WRAP
+    doc->self = rb_data_object_wrap(clas, doc, 0, free_doc_cb);
+#else
     doc->self = rb_data_object_alloc(clas, doc, 0, free_doc_cb);
+#endif
     rb_gc_register_address(&doc->self);
     doc->json = json;
     DATA_PTR(doc->self) = doc;
@@ -1277,7 +1281,11 @@ doc_type(int argc, VALUE *argv, VALUE self) {
 	case T_TRUE:	type = rb_cTrueClass;	break;
 	case T_FALSE:	type = rb_cFalseClass;	break;
 	case T_STRING:	type = rb_cString;	break;
+#ifdef RUBY_INTEGER_UNIFICATION
+	case T_FIXNUM:	type = rb_cInteger;	break;
+#else
 	case T_FIXNUM:	type = rb_cFixnum;	break;
+#endif
 	case T_FLOAT:	type = rb_cFloat;	break;
 	case T_ARRAY:	type = rb_cArray;	break;
 	case T_HASH:	type = rb_cHash;	break;
@@ -1535,6 +1543,7 @@ doc_dump(int argc, VALUE *argv, VALUE self) {
 	    out.buf = buf;
 	    out.end = buf + sizeof(buf) - 10;
 	    out.allocated = 0;
+	    out.omit_nil = oj_default_options.dump_opts.omit_nil;
 	    oj_dump_leaf_to_json(leaf, &oj_default_options, &out);
 	    rjson = rb_str_new2(out.buf);
 	    if (out.allocated) {
@@ -1589,6 +1598,12 @@ doc_close(VALUE self) {
 Oj = rb_define_module("Oj");
 #endif
 
+static VALUE
+doc_not_implemented(VALUE self) {
+    rb_raise(rb_eNotImpError, "Not implemented.");
+    return Qnil;
+}
+
 /* Document-class: Oj::Doc
  *
  * The Doc class is used to parse and navigate a JSON document. The model it
@@ -1653,4 +1668,7 @@ oj_init_doc() {
     rb_define_method(oj_doc_class, "dump", doc_dump, -1);
     rb_define_method(oj_doc_class, "size", doc_size, 0);
     rb_define_method(oj_doc_class, "close", doc_close, 0);
+
+    rb_define_method(oj_doc_class, "clone", doc_not_implemented, 0);
+    rb_define_method(oj_doc_class, "dup", doc_not_implemented, 0);
 }
