@@ -46,6 +46,7 @@ extern "C" {
 #endif
 
 #include "stdint.h"
+#include "stdbool.h"
 
 #if USE_PTHREAD_MUTEX
 #include <pthread.h>
@@ -67,6 +68,10 @@ enum st_retval {ST_CONTINUE = 0, ST_STOP = 1, ST_DELETE = 2, ST_CHECK};
 
 #include "err.h"
 
+#define INF_VAL		"3.0e14159265358979323846"
+#define NINF_VAL	"-3.0e14159265358979323846"
+#define NAN_VAL		"3.3e14159265358979323846"
+
 typedef enum {
     Yes	   = 'y',
     No	   = 'n',
@@ -107,6 +112,14 @@ typedef enum {
     ObjectType	= 'o',
 } DumpType;
 
+typedef enum {
+    AutoNan	= 'a',
+    NullNan	= 'n',
+    HugeNan	= 'h',
+    WordNan	= 'w',
+    RaiseNan	= 'r',
+} NanDump;
+
 typedef enum {
     STRING_IO	= 'c',
     STREAM_IO	= 's',
@@ -114,39 +127,44 @@ typedef enum {
 } StreamWriterType;
 
 typedef struct _DumpOpts {
-    const char	*indent;
-    const char	*before_sep;
-    const char	*after_sep;
-    const char	*hash_nl;
-    const char	*array_nl;
+    bool	use;
+    char	indent_str[16];
+    char	before_sep[16];
+    char	after_sep[16];
+    char	hash_nl[16];
+    char	array_nl[16];
     uint8_t	indent_size;
     uint8_t	before_size;
     uint8_t	after_size;
     uint8_t	hash_size;
     uint8_t	array_size;
+    char	nan_dump;	// NanDump
+    bool	omit_nil;
 } *DumpOpts;
 
 typedef struct _Options {
-    int		indent;		// indention for dump, default 2
-    char	circular;	// YesNo
-    char	auto_define;	// YesNo
-    char	sym_key;	// YesNo
-    char	escape_mode;	// Escape_Mode
-    char	mode;		// Mode
-    char	class_cache;	// YesNo
-    char	time_format;	// TimeFormat
-    char	bigdec_as_num;	// YesNo
-    char	bigdec_load;	// BigLoad
-    char	to_json;	// YesNo
-    char	nilnil;		// YesNo
-    char	allow_gc;	// allow GC during parse
-    char	quirks_mode;	// allow single JSON values instead of documents
-    const char	*create_id;	// 0 or string
-    size_t	create_id_len;	// length of create_id
-    int		sec_prec;	// second precision when dumping time
-    DumpOpts	dump_opts;
-    char	float_prec;	// float precision, linked to float_fmt
-    char	float_fmt[7];	// float format for dumping, if empty use Ruby
+    int			indent;		// indention for dump, default 2
+    char		circular;	// YesNo
+    char		auto_define;	// YesNo
+    char		sym_key;	// YesNo
+    char		escape_mode;	// Escape_Mode
+    char		mode;		// Mode
+    char		class_cache;	// YesNo
+    char		time_format;	// TimeFormat
+    char		bigdec_as_num;	// YesNo
+    char		bigdec_load;	// BigLoad
+    char		to_json;	// YesNo
+    char		nilnil;		// YesNo
+    char		allow_gc;	// allow GC during parse
+    char		quirks_mode;	// allow single JSON values instead of documents
+    char		allow_invalid;	// YesNo - allow invalid unicode
+    const char		*create_id;	// 0 or string
+    size_t		create_id_len;	// length of create_id
+    int			sec_prec;	// second precision when dumping time
+    char		float_prec;	// float precision, linked to float_fmt
+    char		float_fmt[7];	// float format for dumping, if empty use Ruby
+    VALUE		hash_class;	// class to use in place of Hash on load
+    struct _DumpOpts	dump_opts;
 } *Options;
 
 typedef struct _Out {
@@ -160,6 +178,7 @@ typedef struct _Out {
     Options	opts;
     uint32_t	hash_cnt;
     int		allocated;
+    bool	omit_nil;
 } *Out;
 
 typedef struct _StrWriter {
@@ -216,6 +235,7 @@ extern VALUE	oj_object_parse_cstr(int argc, VALUE *argv, char *json, size_t len)
 extern void	oj_parse_options(VALUE ropts, Options copts);
 
 extern void	oj_dump_obj_to_json(VALUE obj, Options copts, Out out);
+extern void	oj_dump_obj_to_json_using_params(VALUE obj, Options copts, Out out, int argc, VALUE *argv);
 extern void	oj_write_obj_to_file(VALUE obj, const char *path, Options copts);
 extern void	oj_write_obj_to_stream(VALUE obj, VALUE stream, Options copts);
 extern void	oj_dump_leaf_to_json(Leaf leaf, Options copts, Out out);
